# -*- coding: utf-8 -*-

import datetime
from django.utils.timezone import now

from zoo.models import EntitySchema, Entity, Job


def test_new_schema(db):

    schema = EntitySchema.objects.create(
        name='person',
        schema={
            'type': 'object',
            'properties': {
                'first_name': {
                    'type': 'string',
                },
                'last_name': {
                    'type': 'string',
                },
                'address': {
                    'type': 'object',
                    'properties': {
                        'street': {
                            'type': 'string',
                        },
                    }
                }
            }
        })
    Entity.objects.create(
        schema=schema,
        meta={},
        content={
            'first_name': 'Leon',
            'last_name': 'Blum',
            'address': {
                'street': 'Rue du Château',
            }
        })
    qs = Entity.objects.content_search(schema, address__street='chateau')
    assert qs.count() == 1


class ActionExample(object):
    def __init__(self, tries=2, content=None):
        self.content = content or {
            'tries': tries,
            'done': False,
        }

    def do(self, **kwargs):
        if self.content['tries'] == 0:
            self.content['done'] = True
            return Job.STATE_SUCCESS
        self.content['tries'] -= 1
        return Job.STATE_ERROR

    def to_json(self):
        return self.content

    @classmethod
    def from_json(cls, content):
        return cls(content=content)


def test_jobs(db):
    assert Job.create(ActionExample()).state == Job.STATE_ERROR
    assert Job.objects.count() == 1
    assert Job.objects.get().content['done'] is False
    assert Job.objects.get().content['tries'] == 1
    assert Job.objects.get().state == Job.STATE_ERROR
    # too fast nothing has been done
    Job.redo()
    assert Job.objects.count() == 1
    assert Job.objects.get().content['done'] is False
    assert Job.objects.get().content['tries'] == 1
    assert Job.objects.get().state == Job.STATE_ERROR
    # first retry
    Job.redo(timestamp=now() + datetime.timedelta(seconds=1))
    assert Job.objects.count() == 1
    assert Job.objects.get().content['done'] is False
    assert Job.objects.get().content['tries'] == 0
    assert Job.objects.get().state == Job.STATE_ERROR
    # second retry
    Job.redo(timestamp=now() + datetime.timedelta(seconds=1))
    assert Job.objects.count() == 1
    assert Job.objects.get().content['done'] is True
    assert Job.objects.get().content['tries'] == 0
    assert Job.objects.get().state == Job.STATE_SUCCESS
